"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TSServerErrorCodes = exports.TSServerStartStatus = exports.TSServerResponse = void 0;
const tsrequest_1 = require("./tsrequest");
const commons_1 = require("./commons");
const data_1 = require("./data");
const tserrors_1 = require("./tserrors");
class TSServerResponse {
    constructor(response, responseParser) {
        this.response = response;
        this.responseParser = responseParser;
    }
    returnData(data, type = 'application/octet-stream', status = tsrequest_1.Resp.OK) { this.response.writeHead(status, { 'Content-Type': type }); this.response.end(_compatibleData(data)); }
    returnString(s, type = 'text/plain', status = tsrequest_1.Resp.OK) { this.response.writeHead(status, { 'Content-Type': type }); this.response.end((0, commons_1.$value)(s, '')); }
    returnObject(v, type = 'application/json', status = tsrequest_1.Resp.OK) {
        if ((0, commons_1.$isdataobject)(v)) {
            throw new tserrors_1.TSHttpError('Impossible to return data as object response', tsrequest_1.Resp.InternalError, undefined, TSServerErrorCodes.ForbiddenDataResponse);
        }
        if ((0, commons_1.$ok)(this.responseParser)) {
            const opts = { errors: [], context: 'json' };
            if (!this.responseParser.validate(v, opts)) {
                throw new tserrors_1.TSHttpError('Invalid structured response', tsrequest_1.Resp.InternalError, { errors: opts.errors }, TSServerErrorCodes.BadResponseStructure);
            }
            v = this.responseParser.rawEncode(v, opts);
        }
        this.response.writeHead(status, { 'Content-Type': type });
        this.response.end(JSON.stringify(v, undefined, 2));
    }
}
exports.TSServerResponse = TSServerResponse;
var TSServerStartStatus;
(function (TSServerStartStatus) {
    TSServerStartStatus[TSServerStartStatus["AlreadyRunning"] = 0] = "AlreadyRunning";
    TSServerStartStatus[TSServerStartStatus["HTTP"] = 1] = "HTTP";
    TSServerStartStatus[TSServerStartStatus["HTTPS"] = 2] = "HTTPS";
})(TSServerStartStatus = exports.TSServerStartStatus || (exports.TSServerStartStatus = {}));
function _compatibleData(data) {
    if (!(0, commons_1.$ok)(data)) {
        return Buffer.from('');
    }
    if (data instanceof Uint8Array || data instanceof ArrayBuffer) {
        return data;
    }
    return (0, data_1.$bufferFromDataLike)(data);
}
var TSServerErrorCodes;
(function (TSServerErrorCodes) {
    TSServerErrorCodes[TSServerErrorCodes["MissingMimeType"] = 10000] = "MissingMimeType";
    TSServerErrorCodes[TSServerErrorCodes["UnknownMethod"] = 10001] = "UnknownMethod";
    TSServerErrorCodes[TSServerErrorCodes["ResourceNotFound"] = 10002] = "ResourceNotFound";
    TSServerErrorCodes[TSServerErrorCodes["MethodNotImplemented"] = 10003] = "MethodNotImplemented";
    TSServerErrorCodes[TSServerErrorCodes["InnaccessibleRoot"] = 10004] = "InnaccessibleRoot";
    TSServerErrorCodes[TSServerErrorCodes["BadParameter"] = 10005] = "BadParameter";
    TSServerErrorCodes[TSServerErrorCodes["BadQueryStructure"] = 10006] = "BadQueryStructure";
    TSServerErrorCodes[TSServerErrorCodes["BadBodyStructure"] = 10007] = "BadBodyStructure";
    TSServerErrorCodes[TSServerErrorCodes["MissingBody"] = 10008] = "MissingBody";
    TSServerErrorCodes[TSServerErrorCodes["BadResponseStructure"] = 10009] = "BadResponseStructure";
    TSServerErrorCodes[TSServerErrorCodes["ForbiddenDataResponse"] = 10010] = "ForbiddenDataResponse";
    TSServerErrorCodes[TSServerErrorCodes["BadJSONBody"] = 10011] = "BadJSONBody";
})(TSServerErrorCodes = exports.TSServerErrorCodes || (exports.TSServerErrorCodes = {}));
//# sourceMappingURL=tsserver_types.js.map